@@ -68,6 +68,7 @@ typedef struct {
 	unsigned int drained:1;
 	unsigned int draining:1;
 	unsigned int xrun_detected:1;
+	unsigned int hw_params_changed:1;
 
 	snd_pcm_uframes_t hw_ptr;
 	snd_pcm_uframes_t boundary;
@@ -213,7 +214,7 @@ static snd_pcm_sframes_t snd_pcm_pipewire_pointer(snd_pcm_ioplug_t *io)
 static int snd_pcm_pipewire_delay(snd_pcm_ioplug_t *io, snd_pcm_sframes_t *delayp)
 {
 	snd_pcm_pipewire_t *pw = io->private_data;
-	int64_t elapsed = 0, filled;
+	int64_t elapsed = 0, filled, avail;
 
 	if (pw->time.rate.num != 0) {
 		struct timespec ts;
@@ -221,20 +222,26 @@ static int snd_pcm_pipewire_delay(snd_pcm_ioplug_t *io, snd_pcm_sframes_t *delay
 		clock_gettime(CLOCK_MONOTONIC, &ts);
 		diff = SPA_TIMESPEC_TO_NSEC(&ts) - pw->time.now;
 	        elapsed = (pw->time.rate.denom * diff) / (pw->time.rate.num * SPA_NSEC_PER_SEC);
-		if (elapsed > pw->time.delay)
-			elapsed = pw->time.delay;
 	}
-	filled = pw->time.delay + snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
+	if (io->stream == SND_PCM_STREAM_PLAYBACK)
+		avail = snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
+	else
+		avail = snd_pcm_ioplug_avail(io, pw->hw_ptr, io->appl_ptr);
+
+	filled = pw->time.delay + avail;
 
 	if (io->stream == SND_PCM_STREAM_PLAYBACK)
 		*delayp = filled - SPA_MIN(elapsed, filled);
 	else
 		*delayp = filled + elapsed;
 
+	pw_log_trace("avail:%"PRIi64" filled %"PRIi64" elapsed:%"PRIi64" delay:%ld %lu %lu",
+			avail, filled, elapsed, *delayp, pw->hw_ptr, io->appl_ptr);
+
 	return 0;
 }
 
-static int
+static snd_pcm_uframes_t
 snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
 		snd_pcm_uframes_t *hw_avail,snd_pcm_uframes_t want)
 {
@@ -245,26 +252,30 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
 	unsigned int channel;
 	struct spa_data *d;
 	void *ptr;
+	uint32_t bl, offset, size;
 
 	d = b->buffer->datas;
 	pwareas = alloca(io->channels * sizeof(snd_pcm_channel_area_t));
 
-	if (io->stream == SND_PCM_STREAM_PLAYBACK) {
-		nframes = d[0].maxsize - SPA_MIN(d[0].maxsize, d[0].chunk->offset);
-		nframes /= pw->stride;
-		nframes = SPA_MIN(nframes, pw->min_avail);
-	} else {
-		nframes = d[0].chunk->size / pw->stride;
+	for (bl = 0; bl < pw->blocks; bl++) {
+		if (io->stream == SND_PCM_STREAM_PLAYBACK) {
+			size = SPA_MIN(d[bl].maxsize, pw->min_avail * pw->stride);
+		} else {
+			offset = SPA_MIN(d[bl].chunk->offset, d[bl].maxsize);
+			size = SPA_MIN(d[bl].chunk->size, d[bl].maxsize - offset);
+		}
+		want = SPA_MIN(want, size / pw->stride);
 	}
-	want = SPA_MIN(nframes, want);
 	nframes = SPA_MIN(want, *hw_avail);
 
 	if (pw->blocks == 1) {
 		if (io->stream == SND_PCM_STREAM_PLAYBACK) {
 			d[0].chunk->size = want * pw->stride;
-			d[0].chunk->offset = 0;
+			d[0].chunk->offset = offset = 0;
+		} else {
+			offset = SPA_MIN(d[0].chunk->offset, d[0].maxsize);
 		}
-		ptr = SPA_PTROFF(d[0].data, d[0].chunk->offset, void);
+		ptr = SPA_PTROFF(d[0].data, offset, void);
 		for (channel = 0; channel < io->channels; channel++) {
 			pwareas[channel].addr = ptr;
 			pwareas[channel].first = channel * pw->sample_bits;
@@ -274,9 +285,11 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
 		for (channel = 0; channel < io->channels; channel++) {
 			if (io->stream == SND_PCM_STREAM_PLAYBACK) {
 				d[channel].chunk->size = want * pw->stride;
-				d[channel].chunk->offset = 0;
+				d[channel].chunk->offset = offset = 0;
+			} else {
+				offset = SPA_MIN(d[channel].chunk->offset, d[channel].maxsize);
 			}
-			ptr = SPA_PTROFF(d[channel].data, d[channel].chunk->offset, void);
+			ptr = SPA_PTROFF(d[channel].data, offset, void);
 			pwareas[channel].addr = ptr;
 			pwareas[channel].first = 0;
 			pwareas[channel].step = pw->sample_bits;
@@ -319,6 +332,7 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
 
 			snd_pcm_areas_silence(pwareas, xfer, io->channels,
 							  frames, io->format);
+			xfer += frames;
 		}
 		if (io->state == SND_PCM_STATE_RUNNING ||
 			io->state == SND_PCM_STATE_DRAINING) {
@@ -326,7 +340,7 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
 			pw->xrun_detected = true;
 		}
 	}
-	return 0;
+	return xfer;
 }
 
 static void on_stream_param_changed(void *data, uint32_t id, const struct spa_pod *param)
@@ -366,7 +380,8 @@ static void on_stream_io_changed(void *data, uint32_t id, void *area, uint32_t s
 	snd_pcm_pipewire_t *pw = data;
 	switch (id) {
 	case SPA_IO_RateMatch:
-		pw->rate_match = area;
+		if (pw->io.stream == SND_PCM_STREAM_PLAYBACK)
+			pw->rate_match = area;
 		break;
 	default:
 		break;
@@ -387,11 +402,17 @@ static void on_stream_process(void *data)
 	snd_pcm_pipewire_t *pw = data;
 	snd_pcm_ioplug_t *io = &pw->io;
 	struct pw_buffer *b;
-	snd_pcm_uframes_t hw_avail, want;
+	snd_pcm_uframes_t hw_avail, before, want, xfer;
 
 	pw_stream_get_time(pw->stream, &pw->time);
 
-	hw_avail = snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
+	if (pw->time.rate.num != 0) {
+		pw->time.delay = pw->time.delay * io->rate * pw->time.rate.num / pw->time.rate.denom;
+		pw->time.rate.denom = io->rate;
+		pw->time.rate.num = 1;
+	}
+
+	before = hw_avail = snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
 
 	if (pw->drained) {
 		pcm_poll_unblock_check(io); /* unblock socket for polling if needed */
@@ -403,9 +424,16 @@ static void on_stream_process(void *data)
 		return;
 
 	want = pw->rate_match ? pw->rate_match->size : hw_avail;
-	pw_log_trace("%p: avail:%lu want:%lu", pw, hw_avail, want);
 
-	snd_pcm_pipewire_process(pw, b, &hw_avail, want);
+	xfer = snd_pcm_pipewire_process(pw, b, &hw_avail, want);
+
+	pw_log_trace("%p: avail-before:%lu avail:%lu want:%lu xfer:%lu",
+			pw, before, hw_avail, want, xfer);
+
+	if (io->stream == SND_PCM_STREAM_PLAYBACK)
+		pw->time.delay += xfer;
+	else
+		pw->time.delay -= SPA_MIN(pw->time.delay, (int64_t)xfer);
 
 	pw_stream_queue_buffer(pw->stream, b);
 
@@ -483,8 +511,9 @@ static int snd_pcm_pipewire_prepare(snd_pcm_ioplug_t *io)
 
 	pw_log_debug("%p: prepare %d %p %lu %ld", pw,
 			pw->error, pw->stream, io->period_size, pw->min_avail);
-	if (pw->error >= 0 && pw->stream != NULL)
+	if (pw->error >= 0 && pw->stream != NULL && !pw->hw_params_changed)
 		goto done;
+	pw->hw_params_changed = false;
 
 	if (pw->stream != NULL) {
 		pw_stream_destroy(pw->stream);
@@ -543,6 +572,8 @@ static int snd_pcm_pipewire_prepare(snd_pcm_ioplug_t *io)
 done:
 	pw->hw_ptr = 0;
 	pw->xrun_detected = false;
+	pw->drained = false;
+	pw->draining = false;
 
 	pw_thread_loop_unlock(pw->main_loop);
 
@@ -703,6 +734,7 @@ static int snd_pcm_pipewire_hw_params(snd_pcm_ioplug_t * io,
 		pw->blocks = 1;
 		pw->stride = (io->channels * pw->sample_bits) / 8;
 	}
+	pw->hw_params_changed = true;
 	pw_log_info("%p: format:%s channels:%d rate:%d stride:%d blocks:%d", pw,
 			spa_debug_type_find_name(spa_type_audio_format, pw->format.format),
 			io->channels, io->rate, pw->stride, pw->blocks);
@@ -766,7 +798,7 @@ static enum snd_pcm_chmap_position channel_to_chmap(enum spa_audio_channel chann
 
 static enum spa_audio_channel chmap_to_channel(enum snd_pcm_chmap_position pos)
 {
-	if (pos < 0 || pos >= SPA_N_ELEMENTS(chmap_info))
+	if (pos >= SPA_N_ELEMENTS(chmap_info))
 		return SPA_AUDIO_CHANNEL_UNKNOWN;
 	return chmap_info[pos].channel;
 }
