@@ -37,7 +37,7 @@
 #include <spa/param/audio/format.h>
 #include <spa/pod/filter.h>
 
-#define NAME "alsa-pcm-source"
+#include "alsa.h"
 
 #include "alsa-pcm.h"
 
@@ -116,7 +116,7 @@ static int impl_node_enum_params(void *object, int seq,
 {
 	struct state *this = object;
 	struct spa_pod *param;
-	uint8_t buffer[1024];
+	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct props *p;
 	struct spa_result_node_params result;
@@ -141,74 +141,76 @@ static int impl_node_enum_params(void *object, int seq,
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_device),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA device"),
+				SPA_PROP_INFO_name, SPA_POD_String(SPA_KEY_API_ALSA_PATH),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA device"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->device, sizeof(p->device)));
 			break;
 		case 1:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_deviceName),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA device name"),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA device name"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->device_name, sizeof(p->device_name)));
 			break;
 		case 2:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_cardName),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA card name"),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA card name"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->card_name, sizeof(p->card_name)));
 			break;
 		case 3:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_minLatency),
-				SPA_PROP_INFO_name, SPA_POD_String("The minimum latency"),
+				SPA_PROP_INFO_description, SPA_POD_String("The minimum latency"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(p->min_latency, 1, INT32_MAX));
 			break;
 		case 4:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_maxLatency),
-				SPA_PROP_INFO_name, SPA_POD_String("The maximum latency"),
+				SPA_PROP_INFO_description, SPA_POD_String("The maximum latency"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(p->max_latency, 1, INT32_MAX));
 			break;
 		case 5:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_PropInfo, id,
-				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_START_CUSTOM),
-				SPA_PROP_INFO_name, SPA_POD_String("Use the driver channelmap"),
-				SPA_PROP_INFO_type, SPA_POD_Bool(p->use_chmap));
-			break;
-		case 6:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_latencyOffsetNsec),
-				SPA_PROP_INFO_name, SPA_POD_String("Latency offset (ns)"),
+				SPA_PROP_INFO_description, SPA_POD_String("Latency offset (ns)"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Long(0LL, 0LL, INT64_MAX));
 			break;
 		default:
-			return 0;
+			param = spa_alsa_enum_propinfo(this, result.index - 6, &b);
+			if (param == NULL)
+				return 0;
 		}
 		break;
 
 	case SPA_PARAM_Props:
+	{
+		struct spa_pod_frame f;
+
 		switch (result.index) {
 		case 0:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_Props, id,
+			spa_pod_builder_push_object(&b, &f,
+                                SPA_TYPE_OBJECT_Props, id);
+			spa_pod_builder_add(&b,
 				SPA_PROP_device,       SPA_POD_Stringn(p->device, sizeof(p->device)),
 				SPA_PROP_deviceName,   SPA_POD_Stringn(p->device_name, sizeof(p->device_name)),
 				SPA_PROP_cardName,     SPA_POD_Stringn(p->card_name, sizeof(p->card_name)),
 				SPA_PROP_minLatency,   SPA_POD_Int(p->min_latency),
 				SPA_PROP_maxLatency,   SPA_POD_Int(p->max_latency),
-				SPA_PROP_START_CUSTOM, SPA_POD_Bool(p->use_chmap),
-				SPA_PROP_latencyOffsetNsec,   SPA_POD_Long(this->process_latency.ns));
+				SPA_PROP_latencyOffsetNsec,   SPA_POD_Long(this->process_latency.ns),
+				0);
+			spa_alsa_add_prop_params(this, &b);
+			param = spa_pod_builder_pop(&b, &f);
 			break;
 		default:
 			return 0;
 		}
 		break;
-
+	}
 	case SPA_PARAM_IO:
 		switch (result.index) {
 		case 0:
@@ -309,6 +311,7 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 	{
 		struct props *p = &this->props;
 		struct spa_process_latency_info info;
+		struct spa_pod *params = NULL;
 
 		if (param == NULL) {
 			reset_props(p);
@@ -323,9 +326,10 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 			SPA_PROP_minLatency,   SPA_POD_OPT_Int(&p->min_latency),
 			SPA_PROP_maxLatency,   SPA_POD_OPT_Int(&p->max_latency),
 			SPA_PROP_latencyOffsetNsec,   SPA_POD_OPT_Long(&info.ns),
-			SPA_PROP_START_CUSTOM, SPA_POD_OPT_Bool(&p->use_chmap));
+			SPA_PROP_params,       SPA_POD_OPT_Pod(&params));
 
 		handle_process_latency(this, &info);
+		spa_alsa_parse_prop_params(this, params);
 
 		emit_node_info(this, false);
 		emit_port_info(this, false);
@@ -586,10 +590,9 @@ static int port_set_format(void *object,
 		if (!this->have_format)
 			return 0;
 
-		spa_alsa_pause(this);
-		clear_buffers(this);
+		spa_log_debug(this->log, "clear format");
 		spa_alsa_close(this);
-		this->have_format = false;
+		clear_buffers(this);
 	} else {
 		struct spa_audio_info info = { 0 };
 
@@ -607,7 +610,6 @@ static int port_set_format(void *object,
 			return err;
 
 		this->current_format = info;
-		this->have_format = true;
 	}
 
 	this->info.change_mask |= SPA_NODE_CHANGE_MASK_PROPS;
@@ -684,7 +686,7 @@ impl_node_port_use_buffers(void *object,
 	if (!this->have_format)
 		return -EIO;
 
-	spa_log_debug(this->log, NAME " %p: use %d buffers", this, n_buffers);
+	spa_log_debug(this->log, "%p: use %d buffers", this, n_buffers);
 
 	if (this->n_buffers > 0) {
 		spa_alsa_pause(this);
@@ -702,7 +704,7 @@ impl_node_port_use_buffers(void *object,
 		b->h = spa_buffer_find_meta_data(b->buf, SPA_META_Header, sizeof(*b->h));
 
 		if (d[0].data == NULL) {
-			spa_log_error(this->log, NAME " %p: need mapped memory", this);
+			spa_log_error(this->log, "%p: need mapped memory", this);
 			return -EINVAL;
 		}
 		spa_list_append(&this->free, &b->link);
@@ -725,7 +727,7 @@ impl_node_port_set_io(void *object,
 
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: io %d %p %zd", this, id, data, size);
+	spa_log_debug(this->log, "%p: io %d %p %zd", this, id, data, size);
 
 	switch (id) {
 	case SPA_IO_Buffers:
@@ -770,7 +772,7 @@ static int impl_node_process(void *object)
 	io = this->io;
 	spa_return_val_if_fail(io != NULL, -EIO);
 
-	spa_log_trace_fp(this->log, NAME " %p; status %d", this, io->status);
+	spa_log_trace_fp(this->log, "%p; status %d", this, io->status);
 
 	if (io->status == SPA_STATUS_HAVE_DATA)
 		return SPA_STATUS_HAVE_DATA;
@@ -793,7 +795,7 @@ static int impl_node_process(void *object)
 	spa_list_remove(&b->link);
 	SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 
-	spa_log_trace_fp(this->log, NAME " %p: dequeue buffer %d", this, b->id);
+	spa_log_trace_fp(this->log, "%p: dequeue buffer %d", this, b->id);
 
 	io->buffer_id = b->id;
 	io->status = SPA_STATUS_HAVE_DATA;
@@ -862,7 +864,6 @@ impl_init(const struct spa_handle_factory *factory,
 	  uint32_t n_support)
 {
 	struct state *this;
-	uint32_t i;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(handle != NULL, -EINVAL);
@@ -873,15 +874,17 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct state *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	alsa_log_topic_init(this->log);
+
 	this->data_system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataSystem);
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 
 	if (this->data_loop == NULL) {
-		spa_log_error(this->log, NAME" %p: a data loop is needed", this);
+		spa_log_error(this->log, "%p: a data loop is needed", this);
 		return -EINVAL;
 	}
 	if (this->data_system == NULL) {
-		spa_log_error(this->log, NAME" %p: a data system is needed", this);
+		spa_log_error(this->log, "%p: a data system is needed", this);
 		return -EINVAL;
 	}
 
@@ -927,40 +930,7 @@ impl_init(const struct spa_handle_factory *factory,
 	spa_list_init(&this->free);
 	spa_list_init(&this->ready);
 
-	for (i = 0; info && i < info->n_items; i++) {
-		const char *k = info->items[i].key;
-		const char *s = info->items[i].value;
-		if (spa_streq(k, SPA_KEY_API_ALSA_PATH)) {
-			snprintf(this->props.device, 63, "%s", s);
-		} else if (spa_streq(k, SPA_KEY_API_ALSA_PCM_CARD)) {
-			this->card_index = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_API_ALSA_OPEN_UCM)) {
-			this->open_ucm = spa_atob(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_CHANNELS)) {
-			this->default_channels = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_RATE)) {
-			this->default_rate = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_FORMAT)) {
-			this->default_format = spa_alsa_format_from_name(s, strlen(s));
-		} else if (spa_streq(k, SPA_KEY_AUDIO_POSITION)) {
-			spa_alsa_parse_position(&this->default_pos, s, strlen(s));
-		} else if (spa_streq(k, "latency.internal.rate")) {
-			this->process_latency.rate = atoi(s);
-		} else if (spa_streq(k, "latency.internal.ns")) {
-			this->process_latency.ns = atoi(s);
-		} else if (spa_streq(k, "api.alsa.period-size")) {
-			this->default_period_size = atoi(s);
-		} else if (spa_streq(k, "api.alsa.headroom")) {
-			this->default_headroom = atoi(s);
-		} else if (spa_streq(k, "api.alsa.disable-mmap")) {
-			this->disable_mmap = spa_atob(s);
-		} else if (spa_streq(k, "api.alsa.disable-batch")) {
-			this->disable_batch = spa_atob(s);
-		} else if (spa_streq(k, "api.alsa.use-chmap")) {
-			this->props.use_chmap = spa_atob(s);
-		}
-	}
-	return spa_alsa_init(this);
+	return spa_alsa_init(this, info);
 }
 
 static const struct spa_interface_info impl_interfaces[] = {
