@@ -43,7 +43,9 @@
 #include <spa/debug/pod.h>
 #include <spa/debug/types.h>
 
-#define NAME "audioconvert"
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT log_topic
+static struct spa_log_topic *log_topic = &SPA_LOG_TOPIC(0, "spa.audioconvert");
 
 #define MAX_PORTS	SPA_AUDIO_MAX_CHANNELS
 
@@ -410,7 +412,7 @@ static int negotiate_link_buffers(struct impl *this, struct link *link)
 static void flush_convert(struct impl *this)
 {
 	int i;
-	spa_log_debug(this->log, NAME " %p: %d", this, this->n_links);
+	spa_log_debug(this->log, "%p: %d", this, this->n_links);
 	for (i = 0; i < this->n_links; i++)
 		this->links[i].io.status = SPA_STATUS_OK;
 }
@@ -419,7 +421,7 @@ static void clean_convert(struct impl *this)
 {
 	int i;
 
-	spa_log_debug(this->log, NAME " %p: %d", this, this->n_links);
+	spa_log_debug(this->log, "%p: %d", this, this->n_links);
 
 	for (i = 0; i < this->n_links; i++)
 		clean_link(this, &this->links[i]);
@@ -430,18 +432,18 @@ static int setup_buffers(struct impl *this, enum spa_direction direction)
 {
 	int i, res;
 
-	spa_log_debug(this->log, NAME " %p: %d %d", this, direction, this->n_links);
+	spa_log_debug(this->log, "%p: %d %d", this, direction, this->n_links);
 
 	if (direction == SPA_DIRECTION_INPUT) {
 		for (i = 0; i < this->n_links; i++) {
 			if ((res = negotiate_link_buffers(this, &this->links[i])) < 0)
-				spa_log_error(this->log, NAME " %p: buffers %d failed %s",
+				spa_log_error(this->log, "%p: buffers %d failed %s",
 						this, i, spa_strerror(res));
 		}
 	} else {
 		for (i = this->n_links-1; i >= 0 ; i--) {
 			if ((res = negotiate_link_buffers(this, &this->links[i])) < 0)
-				spa_log_error(this->log, NAME " %p: buffers %d failed %s",
+				spa_log_error(this->log, "%p: buffers %d failed %s",
 						this, i, spa_strerror(res));
 		}
 	}
@@ -449,6 +451,33 @@ static int setup_buffers(struct impl *this, enum spa_direction direction)
 	return 0;
 }
 
+static int enum_params(struct impl *this,
+				 uint32_t id,
+				 struct spa_result_node_params *result,
+				 const struct spa_pod *filter,
+				 struct spa_pod_builder *builder)
+{
+	int res;
+	result->param = NULL;
+	result->next = result->index;
+	if (result->next < 0x1000) {
+		if (this->fmt[SPA_DIRECTION_INPUT] == this->merger &&
+		    (res = spa_node_enum_params_sync(this->merger,
+				id, &result->next, filter, &result->param, builder)) == 1) {
+			return res;
+		}
+		result->next = 0x1000;
+	}
+	if (result->next >= 0x1000) {
+		result->next &= 0xfff;
+		if ((res = spa_node_enum_params_sync(this->channelmix,
+				id, &result->next, filter, &result->param, builder)) == 1) {
+			result->next |= 0x1000;
+			return res;
+		}
+	}
+	return 0;
+}
 static int impl_node_enum_params(void *object, int seq,
 				 uint32_t id, uint32_t start, uint32_t num,
 				 const struct spa_pod *filter)
@@ -459,6 +488,7 @@ static int impl_node_enum_params(void *object, int seq,
 	uint8_t buffer[1024];
 	struct spa_result_node_params result;
 	uint32_t count = 0;
+	int res;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 	spa_return_val_if_fail(num != 0, -EINVAL);
@@ -466,6 +496,7 @@ static int impl_node_enum_params(void *object, int seq,
 	result.id = id;
 	result.next = start;
       next:
+	res = 0;
 	result.index = result.next++;
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
@@ -522,21 +553,23 @@ static int impl_node_enum_params(void *object, int seq,
 		break;
 
 	case SPA_PARAM_PropInfo:
-		return spa_node_enum_params(this->channelmix, seq, id, start, num, filter);
+		if ((res = enum_params(this, id, &result, filter, &b)) != 1)
+			return res;
+		break;
 
 	case SPA_PARAM_Props:
-		if (this->fmt[SPA_DIRECTION_INPUT] == this->merger)
-			return spa_node_enum_params(this->merger, seq, id, start, num, filter);
-		else
-			return spa_node_enum_params(this->channelmix, seq, id, start, num, filter);
+		if ((res = enum_params(this, id, &result, filter, &b)) != 1)
+			return res;
+		break;
 
 	default:
 		return -ENOENT;
 	}
 
-	if (spa_pod_filter(&b, &result.param, param, filter) < 0)
-		goto next;
-
+	if (res == 0) {
+		if (spa_pod_filter(&b, &result.param, param, filter) < 0)
+			goto next;
+	}
 	spa_node_emit_result(&this->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
 
 	if (++count != num)
@@ -552,7 +585,7 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: io %d %p/%zd", this, id, data, size);
+	spa_log_debug(this->log, "%p: io %d %p/%zd", this, id, data, size);
 
 	switch (id) {
 	case SPA_IO_Position:
@@ -663,7 +696,7 @@ static const struct spa_node_events channelmix_events = {
 	.result = on_node_result,
 };
 
-static const struct spa_node_events resample_events = {
+static const struct spa_node_events proxy_events = {
 	SPA_VERSION_NODE_EVENTS,
 	.result = on_node_result,
 };
@@ -675,7 +708,7 @@ static int reconfigure_mode(struct impl *this, enum spa_param_port_config_mode m
 	struct spa_node *old, *new;
 	bool do_signal;
 
-	spa_log_debug(this->log, NAME " %p: mode %d", this, mode);
+	spa_log_debug(this->log, "%p: mode %d", this, mode);
 
 	/* old node on input/output */
 	old = this->fmt[direction];
@@ -733,7 +766,7 @@ static int reconfigure_mode(struct impl *this, enum spa_param_port_config_mode m
 		spa_pod_builder_init(&b, buffer, sizeof(buffer));
 
 		if (info) {
-			spa_log_debug(this->log, NAME " %p: port config %d", this, info->info.raw.channels);
+			spa_log_debug(this->log, "%p: port config %d", this, info->info.raw.channels);
 			param = spa_format_audio_raw_build(&b, SPA_PARAM_Format, &info->info.raw);
 		}
 		if (mode == SPA_PARAM_PORT_CONFIG_MODE_dsp) {
@@ -884,7 +917,7 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 
 	for (i = 0; i < this->n_nodes; i++) {
 		if ((res = spa_node_send_command(this->nodes[i], command)) < 0) {
-			spa_log_error(this->log, NAME " %p: can't send command to node %d: %s",
+			spa_log_error(this->log, "%p: can't send command to node %d: %s",
 					this, i, spa_strerror(res));
 		}
 	}
@@ -906,7 +939,7 @@ impl_node_add_listener(void *object,
 {
 	struct impl *this = object;
 	struct spa_hook_list save;
-	struct spa_hook l[4];
+	struct spa_hook l[3];
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
@@ -922,18 +955,15 @@ impl_node_add_listener(void *object,
 				&l[0], &fmt_input_events, this);
 	spa_node_add_listener(this->channelmix,
 			&l[1], &channelmix_events, this);
-	spa_node_add_listener(this->resample,
-			&l[2], &resample_events, this);
 	if (this->fmt[SPA_DIRECTION_OUTPUT])
 		spa_node_add_listener(this->fmt[SPA_DIRECTION_OUTPUT],
-				&l[3], &fmt_output_events, this);
+				&l[2], &fmt_output_events, this);
 
 	if (this->fmt[SPA_DIRECTION_INPUT])
 		spa_hook_remove(&l[0]);
 	spa_hook_remove(&l[1]);
-	spa_hook_remove(&l[2]);
 	if (this->fmt[SPA_DIRECTION_OUTPUT])
-		spa_hook_remove(&l[3]);
+		spa_hook_remove(&l[2]);
 
 	this->add_listener = false;
 
@@ -980,7 +1010,7 @@ impl_node_port_enum_params(void *object, int seq,
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 	spa_return_val_if_fail(num != 0, -EINVAL);
 
-	spa_log_debug(this->log, NAME" %p: port %d.%d %d %u", this, direction, port_id, seq, id);
+	spa_log_debug(this->log, "%p: port %d.%d %d %u", this, direction, port_id, seq, id);
 
 	result.id = id;
 	result.next = start;
@@ -1059,7 +1089,7 @@ impl_node_port_set_param(void *object,
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: set param %u on port %d:%d %p",
+	spa_log_debug(this->log, "%p: set param %u on port %d:%d %p",
 				this, id, direction, port_id, param);
 
 	switch (id) {
@@ -1167,7 +1197,7 @@ static int impl_node_process(void *object)
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_trace_fp(this->log, NAME " %p: process %d %d", this, this->n_links, this->n_nodes);
+	spa_log_trace_fp(this->log, "%p: process %d %d", this, this->n_links, this->n_nodes);
 
 	while (1) {
 		res = SPA_STATUS_OK;
@@ -1175,7 +1205,7 @@ static int impl_node_process(void *object)
 		for (i = 0; i < this->n_nodes; i++) {
 			r = spa_node_process(this->nodes[i]);
 
-			spa_log_trace_fp(this->log, NAME " %p: process %d %d: %s",
+			spa_log_trace_fp(this->log, "%p: process %d %d: %s",
 					this, i, r, r < 0 ? spa_strerror(r) : "ok");
 
 			if (SPA_UNLIKELY(r < 0))
@@ -1195,7 +1225,7 @@ static int impl_node_process(void *object)
 			break;
 	}
 
-	spa_log_trace_fp(this->log, NAME " %p: process result: %d", this, res);
+	spa_log_trace_fp(this->log, "%p: process result: %d", this, res);
 
 	return res;
 }
@@ -1298,8 +1328,9 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
-	this->cpu = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_CPU);
+	spa_log_topic_init(this->log, log_topic);
 
+	this->cpu = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_CPU);
 	if (this->cpu)
 		this->max_align = spa_cpu_get_max_align(this->cpu);
 
@@ -1378,8 +1409,8 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_node_add_listener(this->channelmix,
 			&this->listener[0], &channelmix_events, this);
-	spa_node_add_listener(this->resample,
-			&this->listener[1], &resample_events, this);
+	spa_node_add_listener(this->merger,
+			&this->listener[1], &proxy_events, this);
 
 	return 0;
 }
