@@ -48,7 +48,9 @@
 #include "volume-ops.h"
 #include "fmt-ops.h"
 
-#define NAME "merger"
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT log_topic
+static struct spa_log_topic *log_topic = &SPA_LOG_TOPIC(0, "spa.merger");
 
 #define DEFAULT_RATE		48000
 #define DEFAULT_CHANNELS	2
@@ -258,7 +260,7 @@ static int init_port(struct impl *this, enum spa_direction direction, uint32_t p
 	port->format.info.dsp.format = SPA_AUDIO_FORMAT_DSP_F32;
 	spa_list_init(&port->queue);
 
-	spa_log_debug(this->log, NAME " %p: add port %d:%d position:%s",
+	spa_log_debug(this->log, "%p: add port %d:%d position:%s",
 			this, direction, port_id, port->position);
 	emit_port_info(this, port, true);
 
@@ -355,6 +357,15 @@ static int impl_node_enum_params(void *object, int seq,
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Float(p->volume, 0.0, 10.0),
 				SPA_PROP_INFO_container, SPA_POD_Id(SPA_TYPE_Array));
 			break;
+		case 8:
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_PropInfo, id,
+				SPA_PROP_INFO_name, SPA_POD_String("monitor.channel-volumes"),
+				SPA_PROP_INFO_description, SPA_POD_String("Monitor channel volume"),
+				SPA_PROP_INFO_type, SPA_POD_CHOICE_Bool(
+					this->monitor_channel_volumes),
+				SPA_PROP_INFO_params, SPA_POD_Bool(true));
+			break;
 		default:
 			return 0;
 		}
@@ -364,11 +375,13 @@ static int impl_node_enum_params(void *object, int seq,
 	case SPA_PARAM_Props:
 	{
 		struct props *p = &this->props;
+		struct spa_pod_frame f[2];
 
 		switch (result.index) {
 		case 0:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_Props, id,
+			spa_pod_builder_push_object(&b, &f[0],
+                                SPA_TYPE_OBJECT_Props, id);
+			spa_pod_builder_add(&b,
 				SPA_PROP_volume,		SPA_POD_Float(p->volume),
 				SPA_PROP_mute,			SPA_POD_Bool(p->channel.mute),
 				SPA_PROP_channelVolumes,	SPA_POD_Array(sizeof(float),
@@ -388,7 +401,14 @@ static int impl_node_enum_params(void *object, int seq,
 				SPA_PROP_monitorVolumes,	SPA_POD_Array(sizeof(float),
 									SPA_TYPE_Float,
 									p->monitor.n_volumes,
-									p->monitor.volumes));
+									p->monitor.volumes),
+				0);
+			spa_pod_builder_prop(&b, SPA_PROP_params, 0);
+			spa_pod_builder_push_struct(&b, &f[1]);
+			spa_pod_builder_string(&b, "monitor.channel-volumes");
+			spa_pod_builder_bool(&b, this->monitor_channel_volumes);
+			spa_pod_builder_pop(&b, &f[1]);
+			param = spa_pod_builder_pop(&b, &f[0]);
 			break;
 		default:
 			return 0;
@@ -416,7 +436,7 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: io %d %p/%zd", this, id, data, size);
+	spa_log_debug(this->log, "%p: io %d %p/%zd", this, id, data, size);
 
 	switch (id) {
 	case SPA_IO_Position:
@@ -428,12 +448,53 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 	return 0;
 }
 
+static int merger_set_param(struct impl *this, const char *k, const char *s)
+{
+	if (spa_streq(k, "monitor.channel-volumes"))
+		this->monitor_channel_volumes = spa_atob(s);
+	return 0;
+}
+
+static int parse_prop_params(struct impl *this, struct spa_pod *params)
+{
+	struct spa_pod_parser prs;
+	struct spa_pod_frame f;
+
+	spa_pod_parser_pod(&prs, params);
+	if (spa_pod_parser_push_struct(&prs, &f) < 0)
+		return 0;
+
+	while (true) {
+		const char *name;
+		struct spa_pod *pod;
+		char value[512];
+
+		if (spa_pod_parser_get_string(&prs, &name) < 0)
+			break;
+
+		if (spa_pod_parser_get_pod(&prs, &pod) < 0)
+			break;
+
+		if (spa_pod_is_bool(pod)) {
+			snprintf(value, sizeof(value), "%s",
+					SPA_POD_VALUE(struct spa_pod_bool, pod) ?
+					"true" : "false");
+		} else
+			continue;
+
+		spa_log_info(this->log, "key:'%s' val:'%s'", name, value);
+		merger_set_param(this, name, value);
+	}
+	return 0;
+}
+
 static int apply_props(struct impl *this, const struct spa_pod *param)
 {
 	struct spa_pod_prop *prop;
 	struct spa_pod_object *obj = (struct spa_pod_object *) param;
 	struct props *p = &this->props;
 	int changed = 0;
+	uint32_t n;
 
 	SPA_POD_OBJECT_FOREACH(obj, prop) {
 		switch (prop->key) {
@@ -446,32 +507,43 @@ static int apply_props(struct impl *this, const struct spa_pod *param)
 				changed++;
 			break;
 		case SPA_PROP_channelVolumes:
-			if ((p->channel.n_volumes = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
-					p->channel.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0)
+			if ((n = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
+					p->channel.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0) {
+				p->channel.n_volumes = n;
 				changed++;
+			}
 			break;
 		case SPA_PROP_channelMap:
-			if ((p->n_channels = spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
-					p->channel_map, SPA_AUDIO_MAX_CHANNELS)) > 0)
+			if ((n = spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
+					p->channel_map, SPA_AUDIO_MAX_CHANNELS)) > 0) {
+				p->n_channels = n;
 				changed++;
+			}
 			break;
 		case SPA_PROP_softMute:
 			if (spa_pod_get_bool(&prop->value, &p->soft.mute) == 0)
 				changed++;
 			break;
 		case SPA_PROP_softVolumes:
-			if ((p->soft.n_volumes = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
-					p->soft.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0)
+			if ((n = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
+					p->soft.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0) {
+				p->soft.n_volumes = n;
 				changed++;
+			}
 			break;
 		case SPA_PROP_monitorMute:
 			if (spa_pod_get_bool(&prop->value, &p->monitor.mute) == 0)
 				changed++;
 			break;
 		case SPA_PROP_monitorVolumes:
-			if ((p->monitor.n_volumes = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
-					p->monitor.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0)
+			if ((n = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
+					p->monitor.volumes, SPA_AUDIO_MAX_CHANNELS)) > 0) {
+				p->monitor.n_volumes = n;
 				changed++;
+			}
+			break;
+		case SPA_PROP_params:
+			parse_prop_params(this, &prop->value);
 			break;
 		default:
 			break;
@@ -544,7 +616,7 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 		if (this->have_profile && memcmp(&this->format, &info, sizeof(info)) == 0)
 			return 0;
 
-		spa_log_debug(this->log, NAME " %p: port config %d/%d %d", this,
+		spa_log_debug(this->log, "%p: port config %d/%d %d", this,
 				info.info.raw.rate, info.info.raw.channels, monitor);
 
 		for (i = 0; i < this->port_count; i++) {
@@ -632,7 +704,7 @@ impl_node_add_listener(void *object,
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_trace(this->log, NAME" %p: add listener %p", this, listener);
+	spa_log_trace(this->log, "%p: add listener %p", this, listener);
 	spa_hook_list_isolate(&this->hooks, &save, listener, events, data);
 
 	emit_node_info(this, true);
@@ -739,7 +811,7 @@ impl_node_port_enum_params(void *object, int seq,
 	struct port *port;
 	struct spa_pod *param;
 	struct spa_pod_builder b = { 0 };
-	uint8_t buffer[1024];
+	uint8_t buffer[2048];
 	struct spa_result_node_params result;
 	uint32_t count = 0;
 	int res;
@@ -845,7 +917,7 @@ impl_node_port_enum_params(void *object, int seq,
 static int clear_buffers(struct impl *this, struct port *port)
 {
 	if (port->n_buffers > 0) {
-		spa_log_debug(this->log, NAME " %p: clear buffers %p", this, port);
+		spa_log_debug(this->log, "%p: clear buffers %p", this, port);
 		port->n_buffers = 0;
 		spa_list_init(&port->queue);
 	}
@@ -867,7 +939,7 @@ static int setup_convert(struct impl *this)
 	src_fmt = SPA_AUDIO_FORMAT_DSP_F32;
 	dst_fmt = outformat.info.raw.format;
 
-	spa_log_info(this->log, NAME " %p: %s/%d@%dx%d->%s/%d@%d", this,
+	spa_log_info(this->log, "%p: %s/%d@%dx%d->%s/%d@%d", this,
 			spa_debug_type_find_name(spa_type_audio_format, src_fmt),
 			1,
 			informat.info.raw.rate,
@@ -883,7 +955,7 @@ static int setup_convert(struct impl *this)
 				continue;
 			this->src_remap[j] = i;
 			this->dst_remap[i] = j;
-			spa_log_debug(this->log, NAME " %p: channel %d -> %d (%s -> %s)", this,
+			spa_log_debug(this->log, "%p: channel %d -> %d (%s -> %s)", this,
 					i, j,
 					spa_debug_type_find_short_name(spa_type_audio_channel,
 						informat.info.raw.position[i]),
@@ -904,7 +976,7 @@ static int setup_convert(struct impl *this)
 
 	this->is_passthrough = this->conv.is_passthrough;
 
-	spa_log_debug(this->log, NAME " %p: got converter features %08x:%08x passthrough:%d", this,
+	spa_log_debug(this->log, "%p: got converter features %08x:%08x passthrough:%d", this,
 			this->cpu_flags, this->conv.cpu_flags, this->is_passthrough);
 
 	return 0;
@@ -944,7 +1016,7 @@ static int port_set_latency(void *object,
 	enum spa_direction other = SPA_DIRECTION_REVERSE(direction);
 	uint32_t i;
 
-	spa_log_debug(this->log, NAME " %p: set latency direction:%d", this, direction);
+	spa_log_debug(this->log, "%p: set latency direction:%d", this, direction);
 
 	if (latency == NULL) {
 		this->latency[other] = SPA_LATENCY_INFO(other);
@@ -980,7 +1052,7 @@ static int port_set_format(void *object,
 
 	port = GET_PORT(this, direction, port_id);
 
-	spa_log_debug(this->log, NAME " %p: set format", this);
+	spa_log_debug(this->log, "%p: set format", this);
 
 	if (format == NULL) {
 		if (port->have_format) {
@@ -1044,7 +1116,7 @@ static int port_set_format(void *object,
 		}
 		port->format = info;
 
-		spa_log_debug(this->log, NAME " %p: %d %d %d", this,
+		spa_log_debug(this->log, "%p: %d %d %d", this,
 				port_id, port->stride, port->blocks);
 
 		if (!PORT_IS_DSP(direction, port_id))
@@ -1097,7 +1169,7 @@ static void queue_buffer(struct impl *this, struct port *port, uint32_t id)
 {
 	struct buffer *b = &port->buffers[id];
 
-	spa_log_trace_fp(this->log, NAME " %p: queue buffer %d on port %d %d",
+	spa_log_trace_fp(this->log, "%p: queue buffer %d on port %d %d",
 			this, id, port->id, b->flags);
 	if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_QUEUED))
 		return;
@@ -1116,7 +1188,7 @@ static struct buffer *dequeue_buffer(struct impl *this, struct port *port)
 	b = spa_list_first(&port->queue, struct buffer, link);
 	spa_list_remove(&b->link);
 	SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_QUEUED);
-	spa_log_trace_fp(this->log, NAME " %p: dequeue buffer %d on port %d %u",
+	spa_log_trace_fp(this->log, "%p: dequeue buffer %d on port %d %u",
 			this, b->id, port->id, b->flags);
 
 	return b;
@@ -1142,7 +1214,7 @@ impl_node_port_use_buffers(void *object,
 
 	spa_return_val_if_fail(port->have_format, -EIO);
 
-	spa_log_debug(this->log, NAME " %p: use buffers %d on port %d:%d",
+	spa_log_debug(this->log, "%p: use buffers %d on port %d:%d",
 			this, n_buffers, direction, port_id);
 
 	clear_buffers(this, port);
@@ -1158,19 +1230,19 @@ impl_node_port_use_buffers(void *object,
 		b->buf = buffers[i];
 
 		if (n_datas != port->blocks) {
-			spa_log_error(this->log, NAME " %p: invalid blocks %d on buffer %d",
+			spa_log_error(this->log, "%p: invalid blocks %d on buffer %d",
 					this, n_datas, i);
 			return -EINVAL;
 		}
 
 		for (j = 0; j < n_datas; j++) {
 			if (d[j].data == NULL) {
-				spa_log_error(this->log, NAME " %p: invalid memory %d on buffer %d %d %p",
+				spa_log_error(this->log, "%p: invalid memory %d on buffer %d %d %p",
 						this, j, i, d[j].type, d[j].data);
 				return -EINVAL;
 			}
 			if (!SPA_IS_ALIGNED(d[j].data, MAX_ALIGN)) {
-				spa_log_warn(this->log, NAME " %p: memory %d on buffer %d not aligned",
+				spa_log_warn(this->log, "%p: memory %d on buffer %d not aligned",
 						this, j, i);
 			}
 			b->datas[j] = d[j].data;
@@ -1197,7 +1269,7 @@ impl_node_port_set_io(void *object,
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: set io %d on port %d:%d %p",
+	spa_log_debug(this->log, "%p: set io %d on port %d:%d %p",
 			this, id, direction, port_id, data);
 
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
@@ -1234,13 +1306,13 @@ static inline int get_in_buffer(struct impl *this, struct port *port, struct buf
 	struct spa_io_buffers *io;
 
 	if ((io = port->io) == NULL) {
-		spa_log_trace_fp(this->log, NAME " %p: no io on port %d",
+		spa_log_trace_fp(this->log, "%p: no io on port %d",
 				this, port->id);
 		return -EIO;
 	}
 	if (io->status != SPA_STATUS_HAVE_DATA ||
 	    io->buffer_id >= port->n_buffers) {
-		spa_log_trace_fp(this->log, NAME " %p: empty port %d %p %d %d %d",
+		spa_log_trace_fp(this->log, "%p: empty port %d %p %d %d %d",
 				this, port->id, io, io->status, io->buffer_id,
 				port->n_buffers);
 		return -EPIPE;
@@ -1316,7 +1388,7 @@ static int impl_node_process(void *object)
 	spa_return_val_if_fail(outio != NULL, -EIO);
 	spa_return_val_if_fail(this->conv.process != NULL, -EIO);
 
-	spa_log_trace_fp(this->log, NAME " %p: status %p %d %d", this,
+	spa_log_trace_fp(this->log, "%p: status %p %d %d", this,
 			outio, outio->status, outio->buffer_id);
 
 	if (SPA_UNLIKELY((res = get_out_buffer(this, outport, &dbuf)) != 0))
@@ -1353,7 +1425,7 @@ static int impl_node_process(void *object)
 
 		n_samples = SPA_MIN(n_samples, sd->chunk->size / inport->stride);
 
-		spa_log_trace_fp(this->log, NAME " %p: %d %d %d %p", this,
+		spa_log_trace_fp(this->log, "%p: %d %d %d %p", this,
 				sd->chunk->size, maxsize, n_samples, src_datas[i]);
 	}
 
@@ -1382,7 +1454,7 @@ static int impl_node_process(void *object)
 		dd[i].chunk->size = n_samples * outport->stride;
 	}
 
-	spa_log_trace_fp(this->log, NAME " %p: n_src:%d n_dst:%d n_samples:%d max:%d p:%d", this,
+	spa_log_trace_fp(this->log, "%p: n_src:%d n_dst:%d n_samples:%d max:%d p:%d", this,
 			n_src_datas, n_dst_datas, n_samples, maxsize, this->is_passthrough);
 
 	if (!this->is_passthrough)
@@ -1447,7 +1519,7 @@ impl_init(const struct spa_handle_factory *factory,
 {
 	struct impl *this;
 	struct port *port;
-	const char *str;
+	uint32_t i;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(handle != NULL, -EINVAL);
@@ -1458,15 +1530,16 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
-	this->cpu = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_CPU);
+	spa_log_topic_init(this->log, log_topic);
 
+	this->cpu = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_CPU);
 	if (this->cpu)
 		this->cpu_flags = spa_cpu_get_flags(this->cpu);
 
-	this->monitor_channel_volumes = false;
-	if (info) {
-		if ((str = spa_dict_lookup(info, "monitor.channel-volumes")) != NULL)
-			this->monitor_channel_volumes = spa_atob(str);
+	for (i = 0; info && i < info->n_items; i++) {
+		const char *k = info->items[i].key;
+		const char *s = info->items[i].value;
+		merger_set_param(this, k, s);
 	}
 
 	this->latency[SPA_DIRECTION_INPUT] = SPA_LATENCY_INFO(SPA_DIRECTION_INPUT);
