@@ -53,9 +53,16 @@
 
 #include "defs.h"
 
+static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.bluez5.sink.sco");
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
+
+#define DEFAULT_CLOCK_NAME	"clock.system.monotonic"
+
 struct props {
 	uint32_t min_latency;
 	uint32_t max_latency;
+	char clock_name[64];
 };
 
 #define MAX_BUFFERS 32
@@ -130,6 +137,7 @@ struct impl {
 
 	/* Flags */
 	unsigned int started:1;
+	unsigned int following:1;
 
 	/* Sources */
 	struct spa_source source;
@@ -152,8 +160,6 @@ struct impl {
 	uint64_t total_samples;
 };
 
-#define NAME "sco-sink"
-
 #define CHECK_PORT(this,d,p)	((d) == SPA_DIRECTION_INPUT && (p) == 0)
 
 static const uint32_t default_min_latency = MIN_LATENCY;
@@ -164,6 +170,7 @@ static void reset_props(struct props *props)
 {
 	props->min_latency = default_min_latency;
 	props->max_latency = default_max_latency;
+	strncpy(props->clock_name, DEFAULT_CLOCK_NAME, sizeof(props->clock_name));
 }
 
 static int impl_node_enum_params(void *object, int seq,
@@ -243,18 +250,20 @@ static int impl_node_enum_params(void *object, int seq,
 	return 0;
 }
 
-static void set_timeout(struct impl *this, uint64_t timeout)
+static int set_timeout(struct impl *this, uint64_t time)
 {
 	struct itimerspec ts;
-
-	ts.it_value.tv_sec = timeout / SPA_NSEC_PER_SEC;
-	ts.it_value.tv_nsec = timeout % SPA_NSEC_PER_SEC;
+	ts.it_value.tv_sec = time / SPA_NSEC_PER_SEC;
+	ts.it_value.tv_nsec = time % SPA_NSEC_PER_SEC;
 	ts.it_interval.tv_sec = 0;
 	ts.it_interval.tv_nsec = 0;
+	return spa_system_timerfd_settime(this->data_system,
+			this->timerfd, 0, &ts, NULL);
+}
 
-	spa_system_timerfd_settime(this->data_system, this->timerfd, 0, &ts, NULL);
-	this->source.mask = SPA_IO_IN;
-	spa_loop_update_source(this->data_loop, &this->source);
+static int set_timers(struct impl *this)
+{
+	return set_timeout(this, this->following ? 0 : 1);
 }
 
 static uint64_t get_next_timeout(struct impl *this, uint64_t now_time, uint64_t processed_samples)
@@ -273,15 +282,38 @@ static uint64_t get_next_timeout(struct impl *this, uint64_t now_time, uint64_t
 	return next_time;
 }
 
+static int do_reassign_follower(struct spa_loop *loop,
+			bool async,
+			uint32_t seq,
+			const void *data,
+			size_t size,
+			void *user_data)
+{
+	struct impl *this = user_data;
+	set_timers(this);
+	return 0;
+}
+
+static inline bool is_following(struct impl *this)
+{
+	return this->position && this->clock && this->position->clock.id != this->clock->id;
+}
+
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	struct impl *this = object;
+	bool following;
 
-	spa_return_val_if_fail(object != NULL, -EINVAL);
+	spa_return_val_if_fail(this != NULL, -EINVAL);
 
 	switch (id) {
 	case SPA_IO_Clock:
 		this->clock = data;
+		if (this->clock != NULL) {
+			spa_scnprintf(this->clock->name,
+					sizeof(this->clock->name),
+					"%s", this->props.clock_name);
+		}
 		break;
 	case SPA_IO_Position:
 		this->position = data;
@@ -290,6 +322,12 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 		return -ENOENT;
 	}
 
+	following = is_following(this);
+	if (this->started && following != this->following) {
+		spa_log_debug(this->log, "%p: reassign follower %d->%d", this, this->following, following);
+		this->following = following;
+		spa_loop_invoke(this->data_loop, do_reassign_follower, 0, NULL, 0, true, this);
+	}
 	return 0;
 }
 
@@ -343,61 +381,50 @@ static void flush_data(struct impl *this)
 	struct port *port = &this->port;
 	struct spa_data *datas;
 	uint64_t next_timeout = 1;
-	uint32_t min_in_size;
-	uint8_t *packet;
+	const uint32_t min_in_size =
+		(this->transport->codec == HFP_AUDIO_CODEC_MSBC) ?
+		MSBC_DECODED_SIZE : this->transport->write_mtu;
+	uint8_t * const packet =
+		(this->transport->codec == HFP_AUDIO_CODEC_MSBC) ?
+		this->buffer_head : port->write_buffer;
 
 	if (this->transport == NULL || this->transport->sco_io == NULL)
 		return;
 
-	/* get buffer */
-	if (!port->current_buffer) {
-		spa_return_if_fail(!spa_list_is_empty(&port->ready));
-		port->current_buffer = spa_list_first(&port->ready, struct buffer, link);
-		port->ready_offset = 0;
-	}
-	datas = port->current_buffer->buf->datas;
+	if (!spa_list_is_empty(&port->ready)) {
+		/* get buffer */
+		if (!port->current_buffer) {
+			spa_return_if_fail(!spa_list_is_empty(&port->ready));
+			port->current_buffer = spa_list_first(&port->ready, struct buffer, link);
+			port->ready_offset = 0;
+		}
+		datas = port->current_buffer->buf->datas;
 
-	if (this->transport->codec == HFP_AUDIO_CODEC_MSBC) {
-		min_in_size = MSBC_DECODED_SIZE;
-		packet = this->buffer_head;
-	} else {
-		min_in_size = this->transport->write_mtu;
-		packet = port->write_buffer;
-	}
+		/* if buffer has data, copy it into the write buffer */
+		if (datas[0].chunk->size - port->ready_offset > 0) {
+			const uint32_t avail =
+				SPA_MIN(min_in_size, datas[0].chunk->size - port->ready_offset);
+			const uint32_t size =
+				(avail + port->write_buffer_size) > min_in_size ?
+				min_in_size - port->write_buffer_size : avail;
+			memcpy(port->write_buffer + port->write_buffer_size,
+				(uint8_t *)datas[0].data + port->ready_offset,
+				size);
+			port->write_buffer_size += size;
+			port->ready_offset += size;
+		} else {
+			struct buffer *b;
 
-	/* if buffer has data, copy it into the write buffer */
-	if (datas[0].chunk->size - port->ready_offset > 0) {
-		uint32_t avail = SPA_MIN(min_in_size, datas[0].chunk->size - port->ready_offset);
-		uint32_t size = (avail + port->write_buffer_size) > min_in_size ? min_in_size - port->write_buffer_size : avail;
-		memcpy(port->write_buffer + port->write_buffer_size,
-			(uint8_t *)datas[0].data + port->ready_offset,
-			size);
-		port->write_buffer_size += size;
-		port->ready_offset += size;
-	}
+			b = port->current_buffer;
+			port->current_buffer = NULL;
 
-	/* otherwise request a new buffer */
-	else {
-		struct buffer *b;
-
-		b = port->current_buffer;
-		port->current_buffer = NULL;
-
-		/* reuse buffer */
-		spa_list_remove(&b->link);
-		b->outstanding = true;
-		spa_log_trace(this->log, "sco-sink %p: reuse buffer %u", this, b->id);
-		port->io->buffer_id = b->id;
-		spa_node_call_reuse_buffer(&this->callbacks, 0, b->id);
-
-		/* notify we need more data */
-		port->io->status = SPA_STATUS_NEED_DATA;
-		spa_node_call_ready(&this->callbacks, SPA_STATUS_NEED_DATA);
-
-		next_timeout = (this->transport->write_mtu / port->frame_size
-				* SPA_NSEC_PER_SEC / port->current_format.info.raw.rate);
-		set_timeout(this, next_timeout);
-		return;
+			/* reuse buffer */
+			spa_list_remove(&b->link);
+			b->outstanding = true;
+			spa_log_trace(this->log, "sco-sink %p: reuse buffer %u", this, b->id);
+			port->io->buffer_id = b->id;
+			spa_node_call_reuse_buffer(&this->callbacks, 0, b->id);
+		}
 	}
 
 	/* send the data if the write buffer is full */
@@ -476,6 +503,8 @@ static void flush_data(struct impl *this)
 			}
 		}
 
+		spa_log_trace(this->log, "write socket data %d", written);
+
 		next_timeout = get_next_timeout(this, now_time, processed / port->frame_size);
 
 		if (this->clock) {
@@ -510,7 +539,7 @@ static void sco_on_timeout(struct spa_source *source)
 		spa_log_warn(this->log, "error reading timerfd: %s", strerror(errno));
 
 	/* delay if no buffers available */
-	if (spa_list_is_empty(&port->ready)) {
+	if (!this->following && spa_list_is_empty(&port->ready)) {
 		set_timeout(this, this->transport->write_mtu / port->frame_size * SPA_NSEC_PER_SEC / port->current_format.info.raw.rate);
 		port->io->status = SPA_STATUS_NEED_DATA;
 		spa_node_call_ready(&this->callbacks, SPA_STATUS_NEED_DATA);
@@ -547,6 +576,10 @@ static int do_start(struct impl *this)
 	/* Make sure the transport is valid */
 	spa_return_val_if_fail(this->transport != NULL, -EIO);
 
+	this->following = is_following(this);
+
+	spa_log_debug(this->log, "%p: start following:%d", this, this->following);
+
 	/* Do accept if Gateway; otherwise do connect for Head Unit */
 	do_accept = this->transport->profile & SPA_BT_PROFILE_HEADSET_AUDIO_GATEWAY;
 
@@ -586,7 +619,7 @@ static int do_start(struct impl *this)
 	spa_loop_add_source(this->data_loop, &this->source);
 
 	/* start processing */
-	set_timeout(this, 1);
+	set_timers(this);
 
 	/* Set the started flag */
 	this->started = true;
@@ -1049,7 +1082,7 @@ impl_node_port_use_buffers(void *object,
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
 		if (buffers[i]->datas[0].data == NULL) {
-			spa_log_error(this->log, NAME " %p: need mapped memory", this);
+			spa_log_error(this->log, "%p: need mapped memory", this);
 			return -EINVAL;
 		}
 	}
@@ -1107,12 +1140,12 @@ static int impl_node_process(void *object)
 		struct buffer *b = &port->buffers[io->buffer_id];
 
 		if (!b->outstanding) {
-			spa_log_warn(this->log, NAME " %p: buffer %u in use", this, io->buffer_id);
+			spa_log_warn(this->log, "%p: buffer %u in use", this, io->buffer_id);
 			io->status = -EINVAL;
 			return -EINVAL;
 		}
 
-		spa_log_trace(this->log, NAME " %p: queue buffer %u", this, io->buffer_id);
+		spa_log_trace(this->log, "%p: queue buffer %u", this, io->buffer_id);
 
 		spa_list_append(&port->ready, &b->link);
 		b->outstanding = false;
@@ -1225,6 +1258,8 @@ impl_init(const struct spa_handle_factory *factory,
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 	this->data_system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataSystem);
 
+	spa_log_topic_init(this->log, &log_topic);
+
 	if (this->data_loop == NULL) {
 		spa_log_error(this->log, "a data loop is needed");
 		return -EINVAL;
