@@ -109,6 +109,15 @@
  *
  */
 
+/**
+ * .--------.     .---------.     .--------.     .----------.     .-------.
+ * | source | --> | capture | --> |        | --> |  source  | --> |  app  |
+ * '--------'     '---------'     | echo   |     '----------'     '-------'
+ *                                | cancel |
+ * .--------.     .---------.     |        |     .----------.     .--------.
+ * |  app   | --> |  sink   | --> |        | --> | playback | --> |  sink  |
+ * '--------'     '---------'     '--------'     '----------'     '--------'
+ */
 #define NAME "echo-cancel"
 
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
@@ -255,6 +264,8 @@ static void process(struct impl *impl)
 
 	pw_stream_queue_buffer(impl->playback, pout);
 
+	pw_stream_trigger_process(impl->playback);
+
 	/* Now run the canceller */
 	echo_cancel_run(impl->aec_info, impl->aec, rec,	play, out, size / sizeof(float));
 
@@ -309,6 +320,7 @@ static void process(struct impl *impl)
 		spa_ringbuffer_read_update(&impl->out_ring, oindex);
 		avail -= size;
 	}
+	pw_stream_trigger_process(impl->source);
 
 done:
 	impl->sink_ready = false;
@@ -689,7 +701,8 @@ static int setup_streams(struct impl *impl)
 			PW_DIRECTION_OUTPUT,
 			PW_ID_ANY,
 			PW_STREAM_FLAG_MAP_BUFFERS |
-			PW_STREAM_FLAG_RT_PROCESS,
+			PW_STREAM_FLAG_RT_PROCESS |
+			PW_STREAM_FLAG_TRIGGER,
 			params, n_params)) < 0)
 		return res;
 
@@ -706,7 +719,8 @@ static int setup_streams(struct impl *impl)
 			PW_ID_ANY,
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
-			PW_STREAM_FLAG_RT_PROCESS,
+			PW_STREAM_FLAG_RT_PROCESS |
+			PW_STREAM_FLAG_TRIGGER,
 			params, n_params)) < 0)
 		return res;
 
@@ -954,7 +968,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		unsigned int factor = 0;
 		unsigned int new_num = 0;
 
-		sscanf(impl->aec_info->latency, "%u/%u", &num, &denom);
+		spa_assert_se(sscanf(impl->aec_info->latency, "%u/%u", &num, &denom) == 2);
 
 		if ((str = pw_properties_get(props, PW_KEY_NODE_LATENCY)) != NULL) {
 			sscanf(str, "%u/%u", &req_num, &req_denom);
@@ -967,13 +981,9 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 			pw_properties_set(props, PW_KEY_NODE_LATENCY, impl->aec_info->latency);
 			impl->aec_blocksize = sizeof(float) * impl->info.rate * num / denom;
 		} else {
-			char* new_latency_str = (char*)calloc(strlen(str), sizeof(char));
-
-			sprintf(new_latency_str, "%u/%u", new_num, req_denom);
-			pw_log_info("Setting node latency to %s", new_latency_str);
-			pw_properties_set(props, PW_KEY_NODE_LATENCY, new_latency_str);
+			pw_log_info("Setting node latency to %u/%u", new_num, req_denom);
+			pw_properties_setf(props, PW_KEY_NODE_LATENCY, "%u/%u", new_num, req_denom);
 			impl->aec_blocksize = sizeof(float) * impl->info.rate * num / denom * factor;
-			free(new_latency_str);
 		}
 	} else {
 		/* Implementation doesn't care about the block size */
