@@ -49,6 +49,9 @@
 
 #define NAME "filter-chain"
 
+PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
+#define PW_LOG_TOPIC_DEFAULT mod_topic
+
 /**
  * \page page_module_filter_chain PipeWire Module: Filter-Chain
  *
@@ -244,7 +247,7 @@ struct impl {
 	unsigned int do_disconnect:1;
 	unsigned int unloading:1;
 
-	uint32_t rate;
+	long unsigned rate;
 
 	struct graph graph;
 };
@@ -316,6 +319,8 @@ done:
 		pw_stream_queue_buffer(impl->capture, in);
 	if (out != NULL)
 		pw_stream_queue_buffer(impl->playback, out);
+
+	pw_stream_trigger_process(impl->playback);
 }
 
 static float get_default(struct impl *impl, struct descriptor *desc, uint32_t p)
@@ -413,7 +418,6 @@ static struct spa_pod *get_prop_info(struct graph *graph, struct spa_pod_builder
 	spa_pod_builder_push_object(b, &f[0],
 			SPA_TYPE_OBJECT_PropInfo, SPA_PARAM_PropInfo);
 	spa_pod_builder_add (b,
-			SPA_PROP_INFO_id, SPA_POD_Id(SPA_PROP_START_CUSTOM + idx),
 			SPA_PROP_INFO_name, SPA_POD_String(name),
 			0);
 	spa_pod_builder_prop(b, SPA_PROP_INFO_type, 0);
@@ -528,30 +532,8 @@ static void param_props_changed(struct impl *impl, const struct spa_pod *param)
 	int changed = 0;
 
 	SPA_POD_OBJECT_FOREACH(obj, prop) {
-		uint32_t idx;
-		float value;
-		struct port *port;
-
-		if (prop->key == SPA_PROP_params) {
+		if (prop->key == SPA_PROP_params)
 			changed += parse_params(graph, &prop->value);
-			continue;
-		}
-		if (prop->key < SPA_PROP_START_CUSTOM)
-			continue;
-		idx = prop->key - SPA_PROP_START_CUSTOM;
-		if (idx >= graph->n_control)
-			continue;
-
-		if (spa_pod_get_float(&prop->value, &value) < 0)
-			continue;
-
-		port = graph->control_port[idx];
-
-		if (port->control_data != value) {
-			port->control_data = value;
-			changed++;
-			pw_log_info("control %d to %f", idx, port->control_data);
-		}
 	}
 	if (changed > 0) {
 		uint8_t buffer[1024];
@@ -584,6 +566,23 @@ static void param_latency_changed(struct impl *impl, const struct spa_pod *param
 		pw_stream_update_params(impl->playback, params, 1);
 }
 
+static void state_changed(void *data, enum pw_stream_state old,
+		enum pw_stream_state state, const char *error)
+{
+	struct impl *impl = data;
+	struct graph *graph = &impl->graph;
+
+	switch (state) {
+	case PW_STREAM_STATE_PAUSED:
+		pw_stream_flush(impl->playback, false);
+		pw_stream_flush(impl->capture, false);
+		graph_reset(graph);
+		break;
+	default:
+		break;
+	}
+}
+
 static void param_changed(void *data, uint32_t id, const struct spa_pod *param)
 {
 	struct impl *impl = data;
@@ -608,6 +607,7 @@ static const struct pw_stream_events in_stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = capture_destroy,
 	.process = capture_process,
+	.state_changed = state_changed,
 	.param_changed = param_changed
 };
 
@@ -621,6 +621,7 @@ static void playback_destroy(void *d)
 static const struct pw_stream_events out_stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = playback_destroy,
+	.state_changed = state_changed,
 	.param_changed = param_changed
 };
 
@@ -700,7 +701,8 @@ static int setup_streams(struct impl *impl)
 			PW_ID_ANY,
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
-			PW_STREAM_FLAG_RT_PROCESS,
+			PW_STREAM_FLAG_RT_PROCESS  |
+			PW_STREAM_FLAG_TRIGGER,
 			params, n_params);
 	free(b.data);
 
@@ -752,6 +754,11 @@ static struct plugin *plugin_load(struct impl *impl, const char *type, const cha
 	else if (spa_streq(type, "ladspa")) {
 		pl = load_ladspa_plugin(path, NULL);
 	}
+#ifdef HAVE_LILV
+	else if (spa_streq(type, "lv2")) {
+		pl = load_lv2_plugin(path, NULL);
+	}
+#endif
 	if (pl == NULL)
 		goto exit;
 
@@ -1066,7 +1073,7 @@ static int load_node(struct graph *graph, struct spa_json *json)
 
 	if (spa_streq(type, "builtin")) {
 		snprintf(plugin, sizeof(plugin), "%s", "builtin");
-	} else if (!spa_streq(type, "ladspa"))
+	} else if (!spa_streq(type, "ladspa") && !spa_streq(type, "lv2"))
 		return -ENOTSUP;
 
 	pw_log_info("loading type:%s plugin:%s label:%s", type, plugin, label);
@@ -1273,7 +1280,7 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 			sd = dd = NULL;
 
 		for (i = 0; i < n_hndl; i++) {
-			if ((node->hndl[i] = d->instantiate(d, impl->rate, i, node->config)) == NULL) {
+			if ((node->hndl[i] = d->instantiate(d, &impl->rate, i, node->config)) == NULL) {
 				pw_log_error("cannot create plugin instance");
 				res = -ENOMEM;
 				goto error;
@@ -1305,6 +1312,14 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 			graph->n_control++;
 		}
 	}
+	pw_log_info("suggested rate:%lu capture:%d playback:%d", impl->rate,
+			impl->capture_info.rate, impl->playback_info.rate);
+
+	if (impl->capture_info.rate == 0)
+		impl->capture_info.rate = impl->rate;
+	if (impl->playback_info.rate == 0)
+		impl->playback_info.rate = impl->rate;
+
 	/* now collect all input and output ports for all the handles. */
 	for (i = 0; i < n_hndl; i++) {
 		if (inputs == NULL) {
@@ -1640,6 +1655,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct spa_cpu *cpu_iface;
 	int res;
 
+	PW_LOG_TOPIC_INIT(mod_topic);
+
 	impl = calloc(1, sizeof(struct impl));
 	if (impl == NULL)
 		return -errno;
