@@ -222,14 +222,18 @@ process_messages(struct client_data *data)
 
 		marshal = pw_resource_get_marshal(resource);
 		if (marshal == NULL || msg->opcode >= marshal->n_client_methods) {
-			res = -ENOSYS;
-			goto invalid_method;
+			pw_resource_errorf_id(resource, msg->id,
+					-ENOSYS, "invalid method id:%u op:%u",
+					msg->id, msg->opcode);
+			continue;
 		}
 
 		demarshal = marshal->server_demarshal;
 		if (!demarshal[msg->opcode].func) {
-			res = -ENOTSUP;
-			goto invalid_message;
+			pw_resource_errorf_id(resource, msg->id,
+					-ENOTSUP, "function not supported id:%u op:%u",
+					msg->id, msg->opcode);
+			continue;
 		}
 
 		permissions = pw_resource_get_permissions(resource);
@@ -237,31 +241,28 @@ process_messages(struct client_data *data)
 
 		if ((required & permissions) != required) {
 			pw_resource_errorf_id(resource, msg->id,
-				-EACCES, "no permission to call method %u on %u (requires %08x, have %08x)",
-				msg->opcode, msg->id, required, permissions);
+				-EACCES, "no permission to call method %u on %u "
+				"(requires "PW_PERMISSION_FORMAT", have "PW_PERMISSION_FORMAT")",
+				msg->opcode, msg->id,
+				PW_PERMISSION_ARGS(required), PW_PERMISSION_ARGS(permissions));
 			continue;
 		}
 
 		pw_protocol_native_connection_enter(conn);
 		res = demarshal[msg->opcode].func(resource, msg);
 		pw_protocol_native_connection_leave(conn);
-		if (res < 0)
-			goto invalid_message;
+		if (res < 0) {
+			pw_resource_errorf_id(resource, msg->id,
+					res, "invalid message id:%u op:%u (%s)",
+					msg->id, msg->opcode, spa_strerror(res));
+			debug_msg("*invalid message*", msg, true);
+		}
 	}
 	res = 0;
 done:
 	context->current_client = NULL;
 	return res;
 
-invalid_method:
-	pw_resource_errorf_id(resource, msg->id, res, "invalid method id:%u op:%u",
-			msg->id, msg->opcode);
-	goto done;
-invalid_message:
-	pw_resource_errorf_id(resource, msg->id, res, "invalid message id:%u op:%u (%s)",
-			msg->id, msg->opcode, spa_strerror(res));
-	debug_msg("*invalid message*", msg, true);
-	goto done;
 error:
 	if (client->core_resource)
 		pw_resource_errorf(client->core_resource, res, "client error %d (%s)",
@@ -419,9 +420,10 @@ static struct client_data *client_new(struct server *s, int fd)
 	struct pw_impl_client *client;
 	struct pw_protocol *protocol = s->this.protocol;
 	socklen_t len;
-	struct ucred ucred;
 #if defined(__FreeBSD__)
 	struct xucred xucred;
+#else
+	struct ucred ucred;
 #endif
 	struct pw_context *context = protocol->context;
 	struct pw_properties *props;
@@ -477,6 +479,8 @@ static struct client_data *client_new(struct server *s, int fd)
 #endif
 		pw_properties_setf(props, PW_KEY_SEC_UID, "%d", xucred.cr_uid);
 		pw_properties_setf(props, PW_KEY_SEC_GID, "%d", xucred.cr_gid);
+		// this is what Linuxulator does at the moment, see sys/compat/linux/linux_socket.c
+		pw_properties_set(props, PW_KEY_SEC_LABEL, "unconfined");
 	}
 #endif
 
