@@ -39,6 +39,7 @@
 #include "defs.h"
 #include "format.h"
 #include "internal.h"
+#include "log.h"
 #include "module.h"
 
 static void on_module_unload(void *obj, void *data, int res, uint32_t id)
@@ -174,6 +175,7 @@ int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, str
 
 	/* We don't use any incoming format setting and use our native format */
 	spa_zero(*info);
+	info->flags = SPA_AUDIO_FLAG_UNPOSITIONED;
 	info->format = SPA_AUDIO_FORMAT_F32P;
 
 	if ((str = pw_properties_get(props, "channels")) != NULL) {
@@ -199,6 +201,7 @@ int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, str
 			return -EINVAL;
 		}
 		channel_map_to_positions(&map, info->position);
+		info->flags &= ~SPA_AUDIO_FLAG_UNPOSITIONED;
 		pw_properties_set(props, "channel_map", NULL);
 	} else {
 		if (info->channels == 0)
@@ -216,6 +219,8 @@ int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, str
 			for (i = 0; i < info->channels; i++)
 				info->position[i] = SPA_AUDIO_CHANNEL_UNKNOWN;
 		}
+		if (info->position[0] != SPA_AUDIO_CHANNEL_UNKNOWN)
+			info->flags &= ~SPA_AUDIO_FLAG_UNPOSITIONED;
 	}
 
 	if ((str = pw_properties_get(props, "rate")) != NULL) {
@@ -227,6 +232,14 @@ int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, str
 	return 0;
 }
 
+bool module_args_parse_bool(const char *v)
+{
+	if (spa_streq(v, "1") || !strcasecmp(v, "y") || !strcasecmp(v, "t") ||
+	    !strcasecmp(v, "yes") || !strcasecmp(v, "true") || !strcasecmp(v, "on"))
+		return true;
+	return false;
+}
+
 #include "modules/registry.h"
 
 static const struct module_info module_list[] = {
@@ -239,6 +252,7 @@ static const struct module_info module_list[] = {
 	{ "module-native-protocol-tcp", create_module_native_protocol_tcp, },
 	{ "module-pipe-source", create_module_pipe_source, },
 	{ "module-pipe-sink", create_module_pipe_sink, },
+	{ "module-raop-discover", create_module_raop_discover, },
 	{ "module-remap-sink", create_module_remap_sink, },
 	{ "module-remap-source", create_module_remap_source, },
 	{ "module-simple-protocol-tcp", create_module_simple_protocol_tcp, },
